package j3d.loader;

import j3d.DefaultScene3D;
import j3d.Matrix3D;
import j3d.Object3D;
import j3d.Point3D;
import j3d.Position3D;
import j3d.Rotation3D;
import j3d.TexturedTriangle3D;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Hashtable;

public class BIN3DSObject3DLoader implements Object3DLoader
{
	//>------ Primary chunk

	private static final int MAIN3DS = 0x4D4D;

	//>------ Main Chunks

	private static final int EDIT3DS = 0x3D3D;  // this is the start of the editor config
	private static final int KEYF3DS = 0xB000;  // this is the start of the keyframer config

	//>------ sub defines of EDIT3DS

	private static final int EDIT_MATERIAL = 0xAFFF;
	private static final int EDIT_SCALE    = 0x0100;
	private static final int EDIT_CONFIG2  = 0x3E3D;
	private static final int EDIT_VIEW_P1  = 0x7012;
	private static final int EDIT_VIEW_P2  = 0x7011;
	private static final int EDIT_VIEW_P3  = 0x7020;
	private static final int EDIT_VIEW1    = 0x7001;
	private static final int EDIT_BACKGR   = 0x1200;
	private static final int EDIT_AMBIENT  = 0x2100;
	private static final int EDIT_OBJECT   = 0x4000;

	private static final int MAT_BITMAP    = 0x1100;
	private static final int EDIT_UNKNW02  = 0x1201;
	private static final int EDIT_UNKNW03  = 0x1300;
	private static final int EDIT_UNKNW04  = 0x1400;
	private static final int EDIT_UNKNW05  = 0x1420;
	private static final int EDIT_UNKNW06  = 0x1450;
	private static final int EDIT_UNKNW07  = 0x1500;
	private static final int EDIT_UNKNW08  = 0x2200;
	private static final int EDIT_UNKNW09  = 0x2201;
	private static final int EDIT_UNKNW10  = 0x2210;
	private static final int EDIT_UNKNW11  = 0x2300;
	private static final int EDIT_UNKNW12  = 0x2302;
	private static final int EDIT_UNKNW13  = 0x3000;

	//>------ sub defines of EDIT_OBJECT
	private static final int OBJ_TRIMESH   = 0x4100;
	private static final int OBJ_LIGHT     = 0x4600;
	private static final int OBJ_CAMERA    = 0x4700;

	private static final int OBJ_UNKNWN01  = 0x4010;
	private static final int OBJ_UNKNWN02  = 0x4012; //>>---- Could be shadow

	//>------ sub defines of OBJ_CAMERA
	private static final int CAM_UNKNWN01  = 0x4710; 
	private static final int CAM_UNKNWN02  = 0x4720;

	//>------ sub defines of OBJ_LIGHT
	private static final int LIT_OFF       = 0x4620;
	private static final int LIT_SPOT      = 0x4610;
	private static final int LIT_UNKNWN01  = 0x465A;

	//>------ sub defines of OBJ_TRIMESH
	private static final int TRI_VERTEXL   = 0x4110;
	private static final int TRI_FACEL2    = 0x4111;
	private static final int TRI_FACEL1    = 0x4120;
	private static final int TRI_FACETEX   = 0x4130;
	private static final int TRI_TEXVERT   = 0x4140;
	private static final int TRI_SMOOTH    = 0x4150;
	private static final int TRI_LOCAL     = 0x4160;
	private static final int TRI_VISIBLE   = 0x4165;

	//	>------ sub defines of EDIT_MATERIAL
	private static final int MAT_NAME01           = 0xA000; // includes name(see mli doc for materials)
	private static final int MAT_AMBIENT_COL      = 0xA010; // ambient color   [RGB1 and RGB2]
	private static final int MAT_DIFFUSE_COL      = 0xA020; // diffuse color       idem
	private static final int MAT_SPECULAR_COL     = 0xA030; // specular color      idem
	private static final int MAT_SHININESS        = 0xA040; // shininess       [amount of]
	private static final int MAT_SHIN_STRENGTH    = 0xA041; // shin. strength     "
	private static final int MAT_TRANSPARENCY     = 0xA050; // transparency       "
	private static final int MAT_TRANS_FALLOFF    = 0xA052; // trans. falloff     "
	private static final int MAT_REFLECT_BLUR     = 0xA053; // reflect blur       "
	private static final int MAT_TWO_SIDED        = 0xA081; // two sided
	private static final int MAT_TRANS_ADD        = 0xA083; // transparency ADD
	private static final int MAT_SELF_ILLUM       = 0xA084; // self illum      [amount of]
	private static final int MAT_WIRE_ON          = 0xA085; // wire on
	private static final int MAT_WIRE_THICKNESS   = 0xA087; // wire thickness (always present, in case of forced
	private static final int MAT_FACE_MAP         = 0xA088; // face map
	private static final int MAT_TFO_IN           = 0xA08A; // trans. falloff IN
	private static final int MAT_SOFTEN           = 0xA08C; // soften
	private static final int MAT_3DWIRE_THICKNESS = 0xA08e; // 3d wire thickness in units (default is in pix)

	private static final int MAT_TYPE             = 0xA100; // material type   [1=flat 2=gour. 3=phong 4=metal]

	private static final int MAT_MAP_TEXTURE1   = 0xA200;
	private static final int MAT_MAP_SPECULAR   = 0xA204;
	private static final int MAT_MAP_OPACITY    = 0xA210;
	private static final int MAT_MAP_REFLECTION = 0xA220;
	private static final int MAT_MAP_BUMP       = 0xA230;
	private static final int MAT_SOME_TFO       = 0xA240; // some trans. falloff amount (not 0)
	private static final int MAT_SOME_RB        = 0xA250; // some reflect. blur

	private static final int MAT_MAP_AMOUNT_OF  = 0x0030; // amount of
	private static final int MAT_MAP_FILENAME   = 0xA300; // map's file name
	private static final int MAT_MAP_TEXTURE2   = 0xA33A;
	private static final int MAT_MAP_SHININESS  = 0xA33C;
	private static final int MAT_MAP_SELF_ILLUM = 0xA33D;
	private static final int MAT_MAP_TILING     = 0xA351; // map options
	private static final int MAT_MAP_BLUR       = 0xA353; // map filtering blur ( 7% -> 0.07 )
	private static final int MAT_MAP_USCALE     = 0xA354; // 1/U scale
	private static final int MAT_MAP_VSCALE     = 0xA356; // 1/V scale
	private static final int MAT_MAP_UOFFSET    = 0xA358; // U offset
	private static final int MAT_MAP_VOFFSET    = 0xA35A; // V offset
	private static final int MAT_MAP_ANGLE      = 0xA35C; // map rotation angle
	private static final int MAT_MAP_LUM_FIRST  = 0xA360; // lum or alpha tint first color (default=00 00 00)
	private static final int MAT_MAP_LUM_SECOND = 0xA362; // lum or alpha tint secnd color (default=FF FF FF)
	private static final int MAT_MAP_R_CHANNEL  = 0xA364; // RGB tint R channel color (default=FF 00 00)
	private static final int MAT_MAP_G_CHANNEL  = 0xA366; // RGB tint G channel color (default=00 FF 00)
	private static final int MAT_MAP_B_CHANNEL  = 0xA368; // RGB tint B channel color (default=00 00 FF)

	//>>------ sub defs of KEYF3DS

	private static final int KEYF_AMBIENT_NODE_TAG	= 0xB001;  
	private static final int KEYF_OBJECT_NODE_TAG	= 0xB002;    
	private static final int KEYF_CAMERA_NODE_TAG	= 0xB003;    
	private static final int KEYF_TARGET_NODE_TAG	= 0xB004;    
	private static final int KEYF_LIGHT_NODE_TAG	= 0xB005;    
	private static final int KEYF_L_TARGET_NODE_TAG	= 0xB006;    
	private static final int KEYF_SPOTLIGHT_NODE_TAG= 0xB007;    
	private static final int KEYF_SEG				= 0xB008;  
	private static final int KEYF_CURTIME			= 0xB009;  
	private static final int KEYF_HDR				= 0xB00a;  
	private static final int KEYF_NODE_HDR			= 0xB010;  
	private static final int KEYF_INSTANCE_NAME		= 0xB011;    
	private static final int KEYF_PRESCALE			= 0xB012;    
	private static final int KEYF_PIVOT				= 0xB013;  
	private static final int KEYF_BOUNDBOX			= 0xB014;    
	private static final int KEYF_MORPH_SMOOTH		= 0xB015;  
	private static final int KEYF_POS_TRACK_TAG		= 0xB020;  
	private static final int KEYF_ROT_TRACK_TAG		= 0xB021;  
	private static final int KEYF_SCL_TRACK_TAG		= 0xB022;  
	private static final int KEYF_FOV_TRACK_TAG		= 0xB023;  
	private static final int KEYF_ROLL_TRACK_TAG	= 0xB024;  
	private static final int KEYF_COL_TRACK_TAG		= 0xB025;  
	private static final int KEYF_MORPH_TRACK_TAG	= 0xB026;  
	private static final int KEYF_HOT_TRACK_TAG		= 0xB027;  
	private static final int KEYF_FALL_TRACK_TAG	= 0xB028;  
	private static final int KEYF_HIDE_TRACK_TAG	= 0xB029;    
	private static final int KEYF_NODE_ID			= 0xB030;  

	//>>------  these define the different color chunk types
	private static final int COL_RGB = 0x0010;
	private static final int COL_TRU = 0x0011;
	private static final int COL_UNK = 0x0013;
	
	private static final int INT_PERCENTAGE = 0x0030;

	//>>------ defines for viewport chunks

	private static final int TOP       = 0x0001;
	private static final int BOTTOM    = 0x0002;
	private static final int LEFT      = 0x0003;
	private static final int RIGHT     = 0x0004;
	private static final int FRONT     = 0x0005;
	private static final int BACK      = 0x0006;
	private static final int USER      = 0x0007;
	private static final int CAMERA    = 0x0008; // 0xFFFF is the actual code read from file
	private static final int LIGHT     = 0x0009;
//	private static final int DISABLED  = 0x0010;
//	private static final int BOGUS     = 0x0011;

	// others
//	private static final int MAT_UNKNOWN1 = 0x2000;
//	private static final int MAT_UNKNOWN2 = 0x2019;

	private static class BooleanHolder
	{
		boolean value;
	}

	private String name;

/*
	private static final String[] viewports = new String[] {
            "Bogus",
            "Top",
            "Bottom",
            "Left",
            "Right",
            "Front",
            "Back",
            "User",
            "Camera",
            "Light",
            "Disabled"
           };
*/
	
//	private int current_chunk=0;
	private char views_read=0;
	private int numb_faces=0,numb_vertices=0;
	private String temp_name;
	private float temp_float;
	private int offset;

	private float scale = 1.0f;
	
	byte[] bin3ds;
	String path; 
	DefaultScene3D scene;
	
	Hashtable<String,short[][][]> materials;

	public BIN3DSObject3DLoader()
	{
	}
	
	public Object3D loadObject(String fileName)
	{
		return loadObject(new File(fileName));
	}
	
	public Object3D loadObject(File file)
	{
		try
		{
			name = file.getAbsolutePath();
			path = file.getParentFile().getAbsolutePath()+File.separatorChar;
			bin3ds = new byte[(int)file.length()];
			return loadObject(new FileInputStream(file));
		} catch(Exception e) { e.printStackTrace(); }
		return null;
	}

	public Object3D loadObject(InputStream stream) throws IOException
	{
		stream.read(bin3ds);
		offset = 0;
		scene = new DefaultScene3D(name);
		materials = new Hashtable<String,short[][][]>();
		while(ReadPrimaryChunk()==0);
		return scene;
	}

	private void fseek(int idx)
	{
		offset = idx;
	}

	private int ReadChar()
	{
		if (offset==bin3ds.length)
			return -1;
		int b = bin3ds[offset++];
		return(b<0?256:0)+b;
	}

	private int ReadInt()
	{
		int temp = ReadChar();
		if (temp==-1)
			return -1;
		return( temp |(ReadChar() << 8));
	}
	
	private int ReadChunkID()
	{
		int id = ReadInt();
		System.err.println("chunk "+Integer.toHexString(id));
		return id;
	}

	private int ReadLong()
	{
		int temp1=ReadInt(),temp2;
		if (temp1==-1)
			return -1;
		temp2=ReadInt();
		if (temp2==-1)
			return -1;
		return(temp1+(temp2<<16));
	}

	private float ReadFloat()
	{
		int temp = ReadLong();
		if (temp==-1)
			return -1;
		return Float.intBitsToFloat(temp);
	}

	int ReadChunkPointer()
	{
		return(ReadLong());
	}

	int GetChunkPointer()
	{
		return(offset-2); // compensate for the already read Marker
	}

	void ChangeChunkPointer(int temp_pointer)
	{
		fseek(temp_pointer);
	}

	int ReadName()
	{
		int teller=0;
		int letter;
		
		temp_name = "Default name";
		
		letter=ReadChar();
		if(letter==0) return(-1); // dummy object
		temp_name="";
		
		do
		{
			temp_name+=(char)letter;
			teller++;
			letter=ReadChar();
		}
		while((letter!=0) &&(teller<12));
System.err.println("ReadName "+temp_name);
		return(0);
	}

	int ReadLongName()
	{
		int letter;
		
		temp_name = "Default name";
		letter=ReadChar();
		if(letter==0) return(-1); // dummy object

		temp_name="";
		
		do
		{
			temp_name+=(char)letter;
			letter=ReadChar();
		}
		while(letter!=0);
System.err.println("ReadLongName "+temp_name);
		return(0);
	}

	int SkipChunk(int chunk_id)
	{
		int current_pointer;
		int temp_pointer;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
System.err.println("Skipping "+Integer.toHexString(chunk_id)+" "+temp_pointer);
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadFloatChunk()
	{
		int current_pointer;
		int temp_pointer;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		temp_float = ReadFloat();

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadIntChunk()
	{
		int current_pointer;
		int temp_pointer;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		System.err.println("int="+ReadInt());

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadColorChunk()
	{
		int current_pointer;
		int temp_pointer;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		ReadTrueColor();

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadUnknownChunk(int chunk_id)
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6; // 2 id + 4 pointer
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		System.err.println("unknown 0x"+Integer.toHexString(chunk_id));
		while(end_found==false)
		{
			temp_int=ReadChunkID();

			switch(temp_int)
			{
				case MAIN3DS:
				case EDIT3DS:
				case KEYF3DS:
				case EDIT_MATERIAL:
				case EDIT_SCALE:
				case EDIT_CONFIG2:
				case EDIT_VIEW_P1:
				case EDIT_VIEW_P2:
				case EDIT_VIEW_P3:
				case EDIT_VIEW1:
				case EDIT_BACKGR:
				case EDIT_AMBIENT:
				case EDIT_OBJECT:
				case EDIT_UNKNW02:
				case EDIT_UNKNW03:
				case EDIT_UNKNW04:
				case EDIT_UNKNW05:
				case EDIT_UNKNW06:
				case EDIT_UNKNW07:
				case EDIT_UNKNW08:
				case EDIT_UNKNW09:
				case EDIT_UNKNW10:
				case EDIT_UNKNW11:
				case EDIT_UNKNW12:
				case EDIT_UNKNW13:
				case OBJ_TRIMESH:
				case OBJ_LIGHT:
				case OBJ_CAMERA:
				case OBJ_UNKNWN01:
				case OBJ_UNKNWN02:
				case CAM_UNKNWN01:
				case CAM_UNKNWN02:
				case LIT_OFF:
				case LIT_SPOT:
				case LIT_UNKNWN01:
				case TRI_VERTEXL:
				case TRI_FACEL2:
				case TRI_FACEL1:
				case TRI_SMOOTH:
				case TRI_LOCAL:
				case TRI_VISIBLE:
				case COL_TRU:
				case MAT_MAP_TILING:
				case MAT_MAP_BLUR:
				case MAT_MAP_FILENAME:
				case MAT_NAME01:
				case MAT_BITMAP:
				case MAT_AMBIENT_COL:
				case MAT_DIFFUSE_COL:
				case MAT_SPECULAR_COL:
				case MAT_SHININESS:
				case MAT_SHIN_STRENGTH:
				case MAT_TRANSPARENCY:
				case MAT_TRANS_FALLOFF:
				case MAT_REFLECT_BLUR:
				case MAT_TYPE:
				case MAT_SELF_ILLUM:
				case MAT_SOME_TFO:
				case MAT_SOME_RB:
				case MAT_TWO_SIDED:
				case MAT_TRANS_ADD:
				case MAT_WIRE_ON:
				case MAT_FACE_MAP:
				case MAT_TFO_IN:
				case MAT_SOFTEN:
				case MAT_3DWIRE_THICKNESS:
				case MAT_WIRE_THICKNESS:
				case MAT_MAP_TEXTURE1:
				case MAT_MAP_TEXTURE2:
				case MAT_MAP_OPACITY:
				case MAT_MAP_BUMP:
				case MAT_MAP_SPECULAR:
				case MAT_MAP_SHININESS:
				case MAT_MAP_SELF_ILLUM:
				case MAT_MAP_REFLECTION:
				case MAT_MAP_AMOUNT_OF:
				case MAT_MAP_USCALE:
				case MAT_MAP_VSCALE:
				case MAT_MAP_UOFFSET:
				case MAT_MAP_VOFFSET:
				case MAT_MAP_ANGLE:
				case MAT_MAP_LUM_FIRST:
				case MAT_MAP_LUM_SECOND:
				case MAT_MAP_R_CHANNEL:
				case MAT_MAP_G_CHANNEL:
				case MAT_MAP_B_CHANNEL:
				case COL_RGB:
				case COL_UNK:
				case TOP:
				case BOTTOM:
				case LEFT:
				case RIGHT:
				case FRONT:
				case BACK:
				case USER:
				case CAMERA:
				case LIGHT:
				//case DISABLED:
				//case BOGUS:
					tellertje+=ReadUnknownChunk(temp_int);
					break;
				default:
					break;
			}

			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}
		System.err.println("end unknown 0x"+Integer.toHexString(chunk_id));

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadRGBColor()
	{
		float[] rgb_val = new float[3];

		for(int i=0;i<3;i++)
			rgb_val[i] = ReadFloat();
		
		return(12);
	}

	int ReadTrueColor()
	{
		int[] true_c_val = new int[3];
		
		for(int i=0;i<3;i++)
			true_c_val[i]=ReadChar();
		
		return(3);
	}

	int ReadBooleanChunk(BooleanHolder bool)
	{
		int current_pointer;
		int temp_pointer;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		bool.value=ReadChar()!=0;
		
		ChangeChunkPointer(current_pointer+temp_pointer); // move to the new chunk position
		return(temp_pointer);
	}

	int ReadSpotChunk()
	{
		int current_pointer;
		int temp_pointer;
		float[] target = new float[4];
//		float hotspot,falloff;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		target[0] = ReadFloat();
		target[1] = ReadFloat();
		target[2] = ReadFloat();
		/*hotspot = */ReadFloat();
		/*falloff = */ReadFloat();
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadLightChunk()
	{
		boolean end_found=false;
		BooleanHolder bool = new BooleanHolder();
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6; // 2 id + 4 pointer
		float[] light_coors = new float[3];
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		light_coors[0] = ReadFloat();
		light_coors[1] = ReadFloat();
		light_coors[2] = ReadFloat();
		tellertje+=12;
		
		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case LIT_UNKNWN01 :
					tellertje+=SkipChunk(LIT_UNKNWN01);
					break;
				case LIT_OFF      :
					tellertje+=ReadBooleanChunk(bool);
					break;
				case LIT_SPOT     :
					tellertje+=ReadSpotChunk();
					break;
				case COL_RGB      :
					tellertje+=ReadRGBColor();
					break;
				case COL_TRU      :
					tellertje+=ReadTrueColor();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadCameraChunk()
	{
		int current_pointer;
		int temp_pointer;
		float[] camera_eye = new float[3];
		float[] camera_focus  = new float[3];
		//float rotation,lens;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		camera_eye[0] = ReadFloat();
		camera_eye[1] = ReadFloat();
		camera_eye[2] = ReadFloat();
		
		camera_focus[0] = ReadFloat();
		camera_focus[1] = ReadFloat();
		camera_focus[2] = ReadFloat();
		
		/*rotation = */ReadFloat();
		/*lens = */ReadFloat();
		
		if((temp_pointer-38)>0) // this means more chunks are to follow
		{
			if(ReadInt()==CAM_UNKNWN01)
			{
				SkipChunk(CAM_UNKNWN01);
			}
			if(ReadInt()==CAM_UNKNWN02)
			{
				SkipChunk(CAM_UNKNWN02);
			}
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadVerticesChunk(ArrayList<Point3D> vertices)
	{
		int current_pointer;
		int temp_pointer;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		numb_vertices  =ReadInt();
System.err.println("Vertices "+numb_vertices);
		for(int i=0;i<numb_vertices;i++)
		{
			Point3D p = new Point3D(ReadFloat(),ReadFloat(),ReadFloat());
			p.x*=-scale;
			p.y*=scale;
			p.z*=scale;
			vertices.add(p);
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadTexVertChunk(ArrayList<Point3D> texvert)
	{
		int current_pointer;
		int temp_pointer;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		numb_vertices  =ReadInt();

		for(int i=0;i<numb_vertices;i++)
		{
			Point3D p = new Point3D(ReadFloat(),ReadFloat(),0f);
			texvert.add(p);
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadSmoothingChunk()
	{
		int current_pointer;
		int temp_pointer;
		//int smoothing;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		for(int i=0;i<numb_faces;i++)
		{
			/*smoothing=*/ReadLong();
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	@SuppressWarnings("unchecked")
	private void applyTexture(ArrayList<Point3D> texvert,ArrayList<ArrayList<Object>> faces)
	{
		Point3D[] def = new Point3D[] {new Point3D(0,0,0),new Point3D(1,0,0),new Point3D(1,1,0)};
		for(int i=0;i<faces.size();i++)
		{
			ArrayList<Object> f = faces.get(i);
			if (f.size()<4)
				continue;
			ArrayList<Object> att = (ArrayList)f.get(3);
			short[][][] mat = (short[][][])att.get(0);
			for (int j=0;j<3;j++)
			{
				int v = ((Integer)f.get(j)).intValue();
				Point3D p;
				if (v<texvert.size())
					p = (Point3D)texvert.get(v);
				else
					p = def[j];
				int x,y;
				if (p.y>=0)
				{
					x = (int)((p.x)*mat[0].length);
					y = (int)((1-p.y)*mat.length);
				}
				else
				{
					x = (int)((p.x)*mat[0].length);
					y = (int)((-p.y)*mat.length);
				}
				if (x<0) x=0;
				else if (x>=mat[0].length) x = mat[0].length-1; 
				if (y<0) y=0;
				else if (y>=mat.length) y = mat.length-1; 
				att.add(new Integer(x));
				att.add(new Integer(y));
			}
		}
	}

	int ReadFaceTexChunk(ArrayList<Point3D> vertices,ArrayList<ArrayList<Object>> faces)
	{
		int current_pointer;
		int temp_pointer;
		int nb_faces;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		ReadLongName();
		short[][][] mat = (short[][][])materials.get(temp_name);
		if (mat!=null)
		{
			nb_faces = ReadInt();
			
			for(int i=0;i<nb_faces;i++)
			{
				ArrayList<Object> f = faces.get(ReadInt());
				ArrayList<Object> att = new ArrayList<Object>(7);
				att.add(mat);
				f.add(att);
			}
		}

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadFacesChunk(ArrayList<Point3D> vertices,ArrayList<ArrayList<Object>> faces)
	{
		int current_pointer;
		int temp_pointer;
//		int temp_diff;
		int temp_int;
		int tellertje=6; // 2 id + 4 pointer
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		numb_faces     =ReadInt();
		tellertje+=2;
System.err.println("Faces "+numb_faces);
		
		for(int i=0;i<numb_faces;i++)
		{
			ArrayList<Object> face = new ArrayList<Object>();
			/**/
			face.add(new Integer(ReadInt()));
			face.add(new Integer(ReadInt()));
			face.add(new Integer(ReadInt()));
			/**/
			/* inverted
			face.insertget(new Integer(ReadInt()),0);
			face.insertget(new Integer(ReadInt()),0);
			face.insertget(new Integer(ReadInt()),0);
			*/
			faces.add(face);
			/*temp_diff=*/ReadInt();
			tellertje+=8;
		}

		while(tellertje<temp_pointer)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case TRI_FACETEX:
					tellertje+=ReadFaceTexChunk(vertices,faces);
					break;
				case TRI_SMOOTH:
					tellertje+=ReadSmoothingChunk();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			tellertje+=2;
		}

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadTranslationChunk(ArrayList<Point3D> vertices)
	{
		int current_pointer;
		int temp_pointer;
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		float[] mat = new float[16];
		for(int j=0;j<4;j++)
		{
			for(int i=0;i<3;i++)
				mat[j*4+i] = ReadFloat();
			mat[j*4+3] = 0;
		}
		mat[12]=0;
		mat[13]=0;
		mat[14]=0;
		mat[15]=1;
		Matrix3D trans_mat = new Matrix3D(mat);

		for (int i=0;i<vertices.size();i++)
		{
			Point3D p = vertices.get(i);
			trans_mat.multiply(p,p);
		}

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadObjChunk()
	{
		boolean end_found=false;
		BooleanHolder bool=new BooleanHolder();
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6; // 2 id + 4 pointer
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		ArrayList<Point3D> vertices = new ArrayList<Point3D>();
		ArrayList<Point3D> texvert = new ArrayList<Point3D>();
		ArrayList<ArrayList<Object>> faces = new ArrayList<ArrayList<Object>>();

		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case TRI_VERTEXL:
					tellertje+=ReadVerticesChunk(vertices);
					break;
				case TRI_TEXVERT:
					tellertje+=ReadTexVertChunk(texvert);
					break;
				case TRI_FACEL1:
					tellertje+=ReadFacesChunk(vertices,faces);
					break;
				case TRI_FACEL2:
					tellertje+=SkipChunk(temp_int);
					break;
				case TRI_LOCAL:
					tellertje+=ReadTranslationChunk(vertices);
					break;
				case TRI_VISIBLE:
					tellertje+=ReadBooleanChunk(bool);
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}
		applyTexture(texvert,faces);

		ChangeChunkPointer(current_pointer+temp_pointer);

		scene.addObject(new DefaultObject3D("object_"+scene.getObjects().length,vertices,faces),new Position3D(new Point3D(0,0,0),new Rotation3D(0,0,0),1));

		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadObjectChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6; // 2 id + 4 pointer
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		if(ReadName()==-1)
		{
			System.err.println("ReadObjectChunk");
			System.exit(0);
		}

		tellertje+=temp_name.length()+1;

		while(end_found==false)
		{
			temp_int=ReadChunkID();

			switch(temp_int)
			{
				case OBJ_UNKNWN01:
					tellertje+=SkipChunk(OBJ_UNKNWN01);
					break;
				case OBJ_UNKNWN02:
					tellertje+=SkipChunk(OBJ_UNKNWN02);
					break;
				case OBJ_TRIMESH :
					tellertje+=ReadObjChunk();
					break;
				case OBJ_LIGHT   :
					tellertje+=ReadLightChunk();
					break;
				case OBJ_CAMERA  :
					tellertje+=ReadCameraChunk();
					break;
				default:
System.err.println("***"+Integer.toHexString(offset));
					tellertje+=SkipChunk(temp_int);
					break;
			}

			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadBackgrChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6; // 2 id + 4 pointer
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case COL_RGB :
					tellertje+=ReadRGBColor();
					break;
				case COL_TRU :
					tellertje+=ReadTrueColor();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadAmbientChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6; // 2 id + 4 pointer
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case COL_RGB :
					tellertje+=ReadRGBColor();
					break;
				case COL_TRU :
					tellertje+=ReadTrueColor();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int FindCameraChunk()
	{
		int temp_pointer=0;
		
		for(int i=0;i<12;i++)
			ReadInt();

		temp_pointer=11;
		temp_pointer=ReadName();
		
		return(temp_pointer);
	}

	int ReadViewPortChunk()
	{
		int current_pointer;
		int temp_pointer;
		int port,attribs;
		
		views_read++;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		attribs=ReadInt();
		if(attribs==3)
		{
		}
		if(attribs==5)
		{
		}
		
		for(int i=1;i<6;i++) ReadInt(); // read 5 ints to get to the viewport
		
		port=ReadInt();
		if((port==0xFFFF) ||(port==0))
		{
			FindCameraChunk();
			port=CAMERA;
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadViewChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case EDIT_VIEW_P1 :
					tellertje+=ReadViewPortChunk();
					break;
				case EDIT_VIEW_P2 :
					tellertje+=SkipChunk(EDIT_VIEW_P2);
					break;
				case EDIT_VIEW_P3 :
					tellertje+=ReadViewPortChunk();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
			
			if(views_read>3)
				end_found=true;
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	private void loadMat(String key,String mat)
	{
		String[] exts = new String[] {"",".pcx",".tga",".gif",".png",".jpg"};
		BufferedImage img = null;
		String n = null;
		for (int i=0;i<exts.length && img==null;i++)
		{
			n = path+mat+exts[i];
			if ((new File(n)).exists())
				img = DefaultObject3DLoader.loadImage(n);
		}
		if (img==null)
		{
			System.err.println("Can not load material "+mat);
			//System.exit(0);
		}
		else
			materials.put(key,TexturedTriangle3D.getTexture(n,img));
	}

	int ReadBitmapChunk()
	{
		int current_pointer;
		int temp_pointer;
		String matName = temp_name; 

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		if(ReadLongName()==-1)
		{
			System.err.println("ReadBitmap");
			System.exit(0);
		}
		loadMat(matName,temp_name);
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}
	
	int ReadTextureChunk(int chunk_id)
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case MAT_MAP_FILENAME:
					tellertje+=ReadBitmapChunk();
					break;
				case MAT_MAP_TILING:
					tellertje+=ReadIntChunk();
					break;
				case MAT_MAP_BLUR:
					tellertje+=ReadFloatChunk();
					break;
				case INT_PERCENTAGE:
					tellertje+=ReadIntChunk();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadMatDefChunk()
	{
		int current_pointer;
		int temp_pointer;

		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		if(ReadLongName()==-1)
		{
			System.err.println("ReadMatDefChunk");
			System.exit(0);
		}
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadMaterialChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();

		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case MAT_NAME01:
					tellertje+=ReadMatDefChunk();
					break;
				case MAT_AMBIENT_COL:
				case MAT_DIFFUSE_COL:
				case MAT_SPECULAR_COL:
					tellertje+=ReadColorChunk();
					break;
				case INT_PERCENTAGE:
					tellertje+=ReadIntChunk();
					break;
				case MAT_BITMAP:
				case MAT_SHININESS:
				case MAT_SHIN_STRENGTH:
				case MAT_TRANSPARENCY:
				case MAT_TRANS_FALLOFF:
				case MAT_REFLECT_BLUR:
				case MAT_TYPE:
				case MAT_SELF_ILLUM:
				case MAT_SOME_TFO:
				case MAT_SOME_RB:
				case MAT_TWO_SIDED:
				case MAT_TRANS_ADD:
				case MAT_WIRE_ON:
				case MAT_FACE_MAP:
				case MAT_TFO_IN:
				case MAT_SOFTEN:
				case MAT_3DWIRE_THICKNESS:
				case MAT_WIRE_THICKNESS:
				case MAT_MAP_REFLECTION:
				case MAT_MAP_BUMP:
					tellertje+=SkipChunk(temp_int);
					break;
				case MAT_MAP_TEXTURE1:
				case MAT_MAP_TEXTURE2:
					tellertje+=ReadTextureChunk(temp_int);
					break;
				default:
System.err.println("mat chunk 0x"+Integer.toHexString(temp_int));
					tellertje+=SkipChunk(temp_int);
					//tellertje+=ReadUnknownChunk(temp_int);
					break;
			}
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}

		ChangeChunkPointer(current_pointer+temp_pointer);
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadEditChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case EDIT_UNKNW02:
				case EDIT_UNKNW03:
				case EDIT_UNKNW04:
				case EDIT_UNKNW05:
				case EDIT_UNKNW06:
				case EDIT_UNKNW07:
				case EDIT_UNKNW08:
				case EDIT_UNKNW09:
				case EDIT_UNKNW10:
				case EDIT_UNKNW11:
				case EDIT_UNKNW12:
				case EDIT_UNKNW13:
					tellertje+=SkipChunk(temp_int);
					//tellertje+=ReadUnknownChunk(temp_int);
					break;
				case EDIT_MATERIAL:
					tellertje+=ReadMaterialChunk();
					break;
				case EDIT_VIEW1:
					tellertje+=ReadViewChunk();
					break;
				case EDIT_BACKGR   :
					tellertje+=ReadBackgrChunk();
					break;
				case EDIT_AMBIENT  :
					tellertje+=ReadAmbientChunk();
					break;
				case EDIT_OBJECT   :
					tellertje+=ReadObjectChunk();
					break;
				case EDIT_SCALE    :
					tellertje+=ReadFloatChunk();
					scale = temp_float;
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}

		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadKeyfChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		while(end_found==false)
		{
			temp_int = ReadChunkID();
			int temp_ptr = ReadChunkPointer();
			System.err.println(Integer.toHexString(temp_int)+" "+temp_ptr);

			switch(temp_int)
			{
				case KEYF_AMBIENT_NODE_TAG:
				case KEYF_OBJECT_NODE_TAG:
				case KEYF_CAMERA_NODE_TAG:
				case KEYF_TARGET_NODE_TAG:
				case KEYF_LIGHT_NODE_TAG:
				case KEYF_L_TARGET_NODE_TAG:
				case KEYF_SPOTLIGHT_NODE_TAG:
					break;
				case KEYF_SEG:
					ReadLong(); // start
					ReadLong(); // end
					tellertje+=temp_ptr;
					break;
				case KEYF_CURTIME:
					ReadLong(); // curframe
					tellertje+=temp_ptr;
					break;
				case KEYF_HDR:
					ReadInt(); // revision
					ReadLongName(); // filename
					ReadLong(); // animlen
					tellertje+=temp_ptr;
					break;
				case KEYF_NODE_HDR:
					ReadLongName(); // objname
					ReadInt(); // flags1
					ReadInt(); // flags2
					ReadInt(); // hierarchy
					tellertje+=temp_ptr;
					break;
				case KEYF_PIVOT:
					ReadFloat(); // x
					ReadFloat(); // y
					ReadFloat(); // z
					tellertje+=temp_ptr;
					break;
				case KEYF_INSTANCE_NAME:
				case KEYF_PRESCALE:
				case KEYF_BOUNDBOX:
					for (int i=6;i<temp_ptr;i++)
						ReadChar();
					tellertje+=temp_ptr;
					break;
				case KEYF_MORPH_SMOOTH:
					ReadFloat(); // z
					tellertje+=temp_ptr;
					break;
				case KEYF_POS_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					int keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						float x = ReadFloat(); // x
						float y = ReadFloat(); // y
						float z = ReadFloat(); // z
						System.err.println("POS "+x+" "+y+" "+z);
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_ROT_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadFloat(); // rad
						float x = ReadFloat(); // x
						float y = ReadFloat(); // y
						float z = ReadFloat(); // z
						System.err.println("ROT "+x+" "+y+" "+z);
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_SCL_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadFloat(); // x
						ReadFloat(); // y
						ReadFloat(); // z
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_FOV_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadFloat(); // fov
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_ROLL_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadFloat(); // rol
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_COL_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadFloat(); // red
						ReadFloat(); // green
						ReadFloat(); // blue
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_MORPH_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadLongName(); // obj_name
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_HOT_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadFloat(); // ang
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_FALL_TRACK_TAG:
					ReadInt(); // flags
					for (int i=0;i<4;i++)
						ReadInt(); // unknown
					keys = ReadInt();
					ReadInt(); // unknown
					for (int i=0;i<keys;i++)
					{
						ReadInt(); // framenum
						ReadLong(); // unknown
						ReadFloat(); // ang
					}
					tellertje+=temp_ptr;
					break;
				case KEYF_HIDE_TRACK_TAG:
					tellertje+=temp_ptr;
					break;
				case KEYF_NODE_ID:
					ReadInt();
					tellertje+=temp_ptr;
					break;
				case EDIT_VIEW1:
					tellertje+=ReadViewChunk();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}
			tellertje+=6;
			if(tellertje>=temp_pointer)
				end_found=true;
		}
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadMainChunk()
	{
		boolean end_found=false;
		int temp_int;
		int current_pointer;
		int temp_pointer;
		int tellertje=6;
		
		current_pointer=GetChunkPointer();
		temp_pointer   =ReadChunkPointer();
		
		while(end_found==false)
		{
			temp_int=ReadChunkID();
			
			switch(temp_int)
			{
				case KEYF3DS :
					tellertje+=ReadKeyfChunk();
					break;
				case EDIT3DS :
					tellertje+=ReadEditChunk();
					break;
				default:
					tellertje+=SkipChunk(temp_int);
					break;
			}

			tellertje+=2;
			if(tellertje>=temp_pointer)
				end_found=true;
		}
		
		ChangeChunkPointer(current_pointer+temp_pointer); 
		// move to the new chunk position
		return(temp_pointer);
	}

	int ReadPrimaryChunk()
	{
		int version;
		
		if(ReadInt()==MAIN3DS)
		{
			//>---------- find version number
			fseek(28);
			version=ReadChar();
			if(version<3)
			{
				System.err.println("Version "+version);
//				return(1);
			}
			fseek(2);
			ReadMainChunk();
		}
		else
			return(1);
		
		return(0);
	}
}
